MAIN_APP = (
    lambda project_name: f"""
            from fastapi import FastAPI

            app = FastAPI(title=f"{project_name.replace("_"," ").title()}",debug=True)

            @app.get("/")
            def root():
                return {{"msg":"Hello World"}}
"""
)
SETTINGS = """
    secret_key: str
    algorithm: str
    expiray:int
"""

POSTGRES_SETTINGS = """
    postgres_user: str 
    postgres_password: str  #! needed for docker
    postgres_db: str 
    postgres_host: str 
    postgres_port: int 
"""
MYSQL_SETTINGS = """
            mysql_user: str
            mysql_root_password: str  #! needed for docker
            mysql_database: str 
            mysql_host: str  
            mysql_port: int
"""
PYDANTIC_MAIN_SETTINGS = (
    lambda settings: f"""
from pydantic import BaseSettings
from dotenv import load_dotenv

load_dotenv()

class Settings(BaseSettings):
    {settings}

settings = Settings()
"""
)
JWT_ENV_VARIABLES = """
            # ------------------- JWT Configuration -------------------------
            SECRET_KEY = 
            ALGORITHM = 
            EXPIRAY = 
"""
POSTGRES_ENV_VARIABLES = """
            # ------------------- Postgresql Configuration -------------------------
            POSTGRES_USER = 
            POSTGRES_PASSWORD = 
            POSTGRES_DB = 
            POSTGRES_HOST = 
            POSTGRES_PORT = 
"""
MYSQL_ENV_VARIABLES = """
            # ------------------- MySQL Configuration -------------------------
            MYSQL_USER = 
            MYSQL_ROOT_PASSWORD =  
            MYSQL_DATABASE = 
            MYSQL_HOST = 
            MYSQL_PORT =
        """
TESTING = """
            from fastapi.testclient import TestClient
            import pytest
            from app.api.main import app

            client = TestClient(app)
        
            def test_main():
                res = client.get("/")
                assert res.status_code == 200
                assert res.json() == {"msg":"Hello World"}
"""
DOCKERFILE = """
        FROM python:3.9

        WORKDIR /app

        COPY requirements.txt .

        RUN python -m pip install -r requirements.txt 

        EXPOSE 8000

        COPY . /app/

        CMD ["uvicorn","app.api.main:app","--host","0","--port","8000"]
"""

DOCKER_COMPOSE_BASE = (
    lambda database: f"""
        services:
            api:
                build: .
                ports:
                    - 8000:8000
                depends_on:
                    - {database}
                env_file:
                    - ./.env
"""
)
POSTGRES_DOCKER_COMPOSE = """
            postgres:
                image: postgres
                ports:
                    - 5432:5432
                env_file:
                    - ./.env
                volumes:
                - postgres-db:/var/lib/postgresql/data

        volumes:
            postgres-db:
"""
MYSQL_DOCKER_COMPOSE = """
            mysql:
                image: mysql
                ports:
                    - 3306:3306
                env_file:
                    - ./.env
                volumes:
                - mysql-db:/var/lib/mysql/data

        volumes:
            mysql-db:
"""



from typing import Optional
import typer
from rich.console import Console
from pathlib import Path
from .managers import pip_manager
from .options import PackageManager, Database

app = typer.Typer(
    help="Start FastAPI Projects in Lightning Speed",
    # pretty_exceptions_enable=True,
    # pretty_exceptions_show_locals=False,
)
console = Console()


@app.command()
def new(
    project_name: str = typer.Argument(
        default=...,
        metavar="⭐ Project Name",
    ),
    dir: Optional[Path] = typer.Option(
        default=None,
        show_default="Current Working Directory",
        metavar="📁 Directory Path",
        help="Where You Want To Start The Project",
    ),
    package_manager: PackageManager = typer.Option(
        default=PackageManager.pip.value,
        metavar="📦 Package Manager",
        help="Choices are pip , poetry (comming soon)",
    ),
    migrations: bool = typer.Option(
        default=False, metavar="🚀 Alembic Migrations", encoding="utf-8"
    ),
    docker: bool = typer.Option(default=False, metavar="🐋 Docker", encoding="utf-8"),
    testing: bool = typer.Option(default=False, metavar="💉 Tests", encoding="utf-8"),
    database: Database = typer.Option(
        default=Database.postgresql.value,
        metavar="📅 Database",
        help="Choices are sqlite3,postgresql,mysql",
    ),
):

    match package_manager:
        case PackageManager.pip:
            pipmanager = pip_manager.PipManager(
                project_name=project_name,
                migrations=migrations,
                testing=testing,
                database=database,
                docker=docker,
            )
            if dir is not None:
                pipmanager.nav_to_dir(dir)
            pipmanager.create_project()
            pipmanager.init_env()
        case PackageManager.poetry:
            # * Comming Soon
            ...


@app.command()
def info():
    console.print(
        """
    ⚡[yellow bold]FastGen[/], Start FastAPI Projects In Lightning Speed
    built with [bold]typer[/], to help with [green bold]FastAPI[/]

    Developed By Kareem Ebrahim (2022)
    https://github.com/kareemmahlees

    Quickly Generate FastAPI Project Template With Migrations , Docker , Tests, Database and more

    Want to help the project ?
    ⭐ Star it on github : https://github.com/kareemmahlees/fastgen
    """
    )


if __name__ == "__main__":
    app()



from pathlib import Path
import os
from rich.console import Console
from ..options import Database
from .base_manager import Manager
import textwrap
from .. import constants


console = Console()


class PipManager(Manager):
    """
    class to handle pip related operations such as creating folders and files ,
    spawning virtual environments , creating migrations , dockerizing and
    creating env file 
    """
    def __init__(
        self,
        project_name: str,
        migrations: bool,
        testing: bool,
        database: Database,
        docker: bool,
    ) -> None:
        self.project_name = project_name
        self.migrations = migrations
        self.testing = testing
        self.database = database
        self.docker = docker

    def nav_to_dir(self, dir_name: str) -> None:
        """
        executed when user defines a `--dir` value
        ### Raises:
            FileNotFoundError: if directory doesn't exist or invalid name
        """
        dir_path = Path(dir_name)
        if not dir_path.exists():
            raise FileNotFoundError(
                f"Folder {dir_path.name} doesn't exist , please provide a valid folder name"
            )
        os.chdir(dir_path)

    def create_project(self):
        """
        entry point function for creating all sort of things (files , migrations ...)
        """
        console.print("Creating Project Structure ....", end="\r")
        self.project_path = Path(self.project_name)
        os.makedirs(self.project_path)

        # create test stuff
        if self.testing:
            test_path = self.project_path / "test"
            os.makedirs(test_path)
            self.generate_test_files(testing_path=test_path)

        # create api stuff
        api_path = self.project_path / "app" / "api"
        os.makedirs(api_path)
        self.generate_main_files(api_path=api_path)

        # create settings stuff
        settings_path = self.project_path / "app" / "core"
        os.makedirs(settings_path)
        self.generate_settings_related_files(settings_path=settings_path)

        # create database stuff
        database_path = self.project_path / "app" / "database"
        os.makedirs(database_path)
        self.generate_db_related_files(database_path=database_path)

        # create .env file
        self.generate_env_file(project_path=self.project_path)

        # create docker file
        if self.docker:
            self.generate_docker_file(project_path=self.project_path)

        console.print("Creating Project Structure .... [green bold]SUCCESS[/]")
        os.chdir(self.project_path)

        # create migrations if True
        # migrations is put here due to weired behaviour of
        # alembic ini file which is created outside the project folder
        if self.migrations:
            os.system("alembic init migrations")

    def init_env(self):
        """
        spawn a pip virtual environment inside the directory when `--package-manager` is set to `pip`
        and installing the required dependencies (fastapi, pytest, alembic, python-dotenv)
        """
        console.print("Spawning Virtual Environment .... ", end="\r")
        os.system("python -m venv .venv")
        console.print("Spawning Virtual Environment .... [green bold]SUCCESS[/]")

        console.print("Installing required dependencies ....", end="\r")
        os.system(f".\.venv\Scripts\python.exe -m pip install --upgrade pip -q")
        os.system(
            f".\.venv\Scripts\python.exe -m pip install fastapi python-dotenv\
            { 'pytest' if self.testing else ''} \
            {'alembic' if self.migrations else ''} -q"
        )
        console.print("Installing required dependencies .... [green bold]SUCCESS[/]")

    def generate_main_files(self, api_path: Path):
        """
        generates main files for the project (main, deps, utils and schemas.py)
        """
        with open(api_path / "__init__.py", "x") as f:
            ...
        with open(api_path / "main.py", "x") as f:
            f.write(textwrap.dedent(constants.MAIN_APP(project_name=self.project_name)))

        with open(api_path / "deps.py", "x") as f:
            ...
        with open(api_path / "utils.py", "x") as f:
            ...
        with open(api_path / "schemas.py", "x") as f:
            f.write(
                textwrap.dedent(
                    """
                from pydantic import BaseModel
                """
                )
            )

    def generate_db_related_files(self, database_path: Path):
        """
        creates database.py file + db.sqlite file when `--database` is set to sqlite3
        """
        with open(database_path / "__init__.py", "x") as f:
            ...
        with open(database_path / "database.py", "x") as f:
            ...
        # * will be added when orms are supported
        # with open(databse_path / "models.py", "x") as f:
        #     ...
        if self.database.value == "sqlite3":
            with open(database_path / "db.sqlite", "x") as f:
                ...

    def generate_settings_related_files(self, settings_path: Path):
        """
        creates the config.py file with a class `Settings` to store environment variables
        """
        with open(settings_path / "__init__.py", "x",encoding="utf-8") as f:
            ...
        with open(settings_path / "config.py", "x",encoding="utf-8") as f:
            settings = constants.SETTINGS
            if self.database.value == "postgresql":
                settings += constants.POSTGRES_SETTINGS
            elif self.database.value == "mysql":
                settings += constants.MYSQL_SETTINGS
            f.write(constants.PYDANTIC_MAIN_SETTINGS(settings))

    def generate_env_file(self, project_path: Path):
        """
        generate env file which contains all environment variables
        """
        jwt_env_variables = constants.JWT_ENV_VARIABLES
        if self.database.value == "postgresql":
            jwt_env_variables += constants.POSTGRES_ENV_VARIABLES
        elif self.database.value == "mysql":
            jwt_env_variables += constants.MYSQL_ENV_VARIABLES
        with open(project_path / ".env", "x",encoding="utf-8") as f:
            f.write(textwrap.dedent(jwt_env_variables))

    def generate_test_files(self, testing_path: Path):
        "generate testing files"
        with open(testing_path / "__init__.py", "x",encoding="utf-8") as f:
            ...
        with open(testing_path / "test_main.py", "x",encoding="utf-8") as f:
            f.write(textwrap.dedent(constants.TESTING))

    def generate_docker_file(self, project_path: Path):
        """
        generate Dockerfile and docker-compose file with respect to the database of choice  
        """
        with open(project_path / "Dockerfile", "x",encoding="utf-8") as f:
            f.write(textwrap.dedent(constants.DOCKERFILE))
        if self.database.value != "sqlite3":
            with open(project_path/"docker-compose.yml","x",encoding="utf-8") as f:
                if self.database.value == "postgresql":
                    docker_compose_base = constants.DOCKER_COMPOSE_BASE("postgres")
                    docker_compose_base += constants.POSTGRES_DOCKER_COMPOSE
                    f.write(textwrap.dedent(docker_compose_base))

                elif self.database.value == "mysql":
                    docker_compose_base = constants.DOCKER_COMPOSE_BASE("mysql")
                    docker_compose_base += constants.MYSQL_DOCKER_COMPOSE
                    f.write(textwrap.dedent(docker_compose_base))





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fastgen', 'fastgen.managers']

package_data = \
{'': ['*']}

install_requires = \
['click==8.1.3',
 'pre-commit>=2.20.0,<3.0.0',
 'rich>=12.5.1,<13.0.0',
 'typer[all]>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['fastgen = fastgen.main:app']}

setup_kwargs = {
    'name': 'fastgen',
    'version': '0.1.2',
    'description': 'FastGen, Start FastAPI Projects In Lightning Speed',
    'long_description': '# ⚡ _**FastGen**_\n\nStart FastAPI Projects in Lightning Speed\n\nBuilt With **Typer** To Help With <span style="color:green">**FastAPI**</span>.\n\n## 👀 **Take A Look**\n\nthis is a glanc of the project structure you will have once you use **FastGen**\n\n![dirs_images](./docs/dir.png)\n\n## ✨ **Installation**\n\n```console\n$ python -m pip install fastgen\n```\n\n**Usage**:\n\n```console\n$ fastgen [OPTIONS] COMMAND [ARGS]...\n```\n\n**Options**:\n\n- `--install-completion`: Install completion for the current shell.\n- `--show-completion`: Show completion for the current shell, to copy it or customize the installation.\n- `--help`: Show this message and exit.\n\n**Commands**:\n\n- `info`\n- `new`\n\n## `fastgen info`\n\n**Usage**:\n\n```console\n$ fastgen info [OPTIONS]\n```\n\n**Options**:\n\n- `--help`: Show this message and exit.\n\n## `fastgen new`\n\n**Usage**:\n\n```console\n$ fastgen new [OPTIONS] ⭐ Project Name\n```\n\n**Arguments**:\n\n- `⭐ Project Name`: [required]\n\n**Options**:\n\n- `--dir 📁 Directory Path`\n- `--package-manager 📦 Package Manager`: [default: pip]\n  ( Options are pip , poetry "Comming Soon" )\n- `--migrations / --no-migrations`: [default: False]\n- `--docker / --no-docker`: [default: False]\n- `--testing / --no-testing`: [default: False]\n- `--database 📅 Database`: [default: postgresql] ( Options are postgresql,mysql,sqlite )\n- `--help`: Show this message and exit.\n\n## 🪲 **Encountered A Problem !**\n\nfeel free to open an issue discussing the problem you faced\n\n## 🤝🏻 **Contributing**\n\nplease refer to [Contribution Guide](./CONTRIBUTING.md)\n',
    'author': 'kareem',
    'author_email': 'kareemmahlees@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kareemmahlees/fastgen',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from abc import ABC, abstractmethod


class Manager(ABC):
    @abstractmethod
    def nav_to_dir(self):
        ...

    @abstractmethod
    def create_project(self):
        ...

    @abstractmethod
    def init_env(self):
        ...
